@@ -268,7 +268,7 @@ export class Builder {
      */
     nested(obj: Offset): void {
       if (obj != this.offset()) {
-        throw new Error('FlatBuffers: struct must be serialized inline.');
+        throw new TypeError('FlatBuffers: struct must be serialized inline.');
       }
     }
   
@@ -278,7 +278,7 @@ export class Builder {
      */
     notNested(): void {
       if (this.isNested) {
-        throw new Error('FlatBuffers: object serialization must not be nested.');
+        throw new TypeError('FlatBuffers: object serialization must not be nested.');
       }
     }
   
@@ -429,7 +429,7 @@ export class Builder {
         this.prep(this.minalign, SIZEOF_INT +
           FILE_IDENTIFIER_LENGTH + size_prefix);
         if (file_identifier.length != FILE_IDENTIFIER_LENGTH) {
-          throw new Error('FlatBuffers: file identifier must be length ' +
+          throw new TypeError('FlatBuffers: file identifier must be length ' +
             FILE_IDENTIFIER_LENGTH);
         }
         for (let i = FILE_IDENTIFIER_LENGTH - 1; i >= 0; i--) {
@@ -458,11 +458,12 @@ export class Builder {
     requiredField(table: Offset, field: number): void {
       const table_start = this.bb.capacity() - table;
       const vtable_start = table_start - this.bb.readInt32(table_start);
-      const ok = this.bb.readInt16(vtable_start + field) != 0;
+      const ok = field < this.bb.readInt16(vtable_start) &&
+                 this.bb.readInt16(vtable_start + field) != 0;
   
       // If this fails, the caller will show what field needs to be set.
       if (!ok) {
-        throw new Error('FlatBuffers: field ' + field + ' must be set');
+        throw new TypeError('FlatBuffers: field ' + field + ' must be set');
       }
     }
   
@@ -549,7 +550,7 @@ export class Builder {
      * 
      * @returns offset of obj
      */
-    createObjectOffset(obj: string | any): Offset {
+    createObjectOffset(obj: string | IGeneratedObject | null): Offset {
       if(obj === null) {
         return 0
       }
@@ -566,7 +567,7 @@ export class Builder {
      * 
      * @returns list of offsets of each non null object
      */
-    createObjectOffsetList(list: string[] | any[]): Offset[] {
+    createObjectOffsetList(list: (string | IGeneratedObject)[]): Offset[] {
       const ret: number[] = [];
   
       for(let i = 0; i < list.length; ++i) {
@@ -575,7 +576,7 @@ export class Builder {
         if(val !== null) {
           ret.push(this.createObjectOffset(val));
         } else {
-          throw new Error(
+          throw new TypeError(
             'FlatBuffers: Argument for createObjectOffsetList cannot contain null.'); 
         }
       }
@@ -583,7 +584,7 @@ export class Builder {
       return ret;
     }
   
-    createStructOffsetList(list: string[] | any[], startFunc: (builder: Builder, length: number) => void): Offset {
+    createStructOffsetList(list: (string | IGeneratedObject)[], startFunc: (builder: Builder, length: number) => void): Offset {
       startFunc(this, list.length);
       this.createObjectOffsetList(list.slice().reverse());
       return this.endVector();
